package as.adamsmith.etherealdialpad;

import android.content.Context;
import android.graphics.Canvas;
import android.graphics.Color;
import android.graphics.Paint;
import android.graphics.Paint.Style;
import android.os.Bundle;
import android.os.RemoteException;
import android.view.MotionEvent;
import android.view.SurfaceView;

public class GridPad extends Pad {	
    GridView gridView;
	
	@Override
	public void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
		
		gridView = new GridView(this);
		setContentView(gridView);
	}

    @Override
    protected void onSynthServiceConnected() {
        gridView.onConnected();
    }

	class GridView extends SurfaceView {
	    int sw = 0, sh = 0;
	    int nx, ny;
	    float[] scaleX, scaleY;
	    int[] colsX, colsY;
		float u, v;
		Paint paint;
		float[] hsv = new float[3];
		boolean touching;
		float[] scale;
		int octaves;
		boolean duet;
		
		public GridView(Context context) {
			super(context);
			paint = new Paint();
			setBackgroundColor(0xFF404040);
		}
		
		@Override
		protected void onSizeChanged(int w, int h, int ow, int oh) {
		    sw = w;
		    sh = h;
		}

        /**
         * We are bound to the synth service.
         */
        void onConnected() {
            try {
            	scale = synthService.getScale();
            	octaves = synthService.getOctaves();
            	duet = synthService.isDuet();
            	
                nx = scale==null?0:(scale.length*octaves+1);
                ny = duet?(scale==null?0:nx):0;
                
                scaleX = scale;
                if (scaleX == null)
                    colsX = null;
                else {
                    colsX = new int[scaleX.length];
                    for (int i = 0; i < scaleX.length; ++i)
                        colsX[i] = noteColor(scaleX[i]);
                }
                
                scaleY = duet?scale:null;
                if (scaleY == null)
                    colsY = null;
                else {
                    colsY = new int[scaleY.length];
                    for (int i = 0; i < scaleY.length; ++i)
                        colsY[i] = noteColor(scaleY[i]);
                }
            } catch (RemoteException e) {
                nx = ny = 0;
                scaleX = scaleY = null;
                colsX = colsY = null;
            }
        }

		@Override
		public boolean onTouchEvent(MotionEvent event) {
			u = event.getX() / sw;
			v = 1.0f - event.getY() / sh;
			
			if(event.getAction() == MotionEvent.ACTION_DOWN) {
				touching = true;
				try {
					synthService.primaryOn();
					synthService.primaryXY(u, v);
				} catch (RemoteException e) {
					e.printStackTrace();
				}
			}
			
			if(event.getAction() == MotionEvent.ACTION_UP) {
				touching = false;
				try {
					synthService.primaryOff();
					synthService.primaryXY(u, v);
				} catch (RemoteException e) {
					e.printStackTrace();
				}
			}
			
			if(event.getAction() == MotionEvent.ACTION_MOVE) {
				try {
					synthService.primaryXY(u, v);
				} catch (RemoteException e) {
					e.printStackTrace();
				}
			}
			
			invalidate();
			
			return true;
		}
		
		@Override
		public void onDraw(Canvas canvas) {
		    // Draw a background colour based on the most recent hue.
		    float r = 128f + 128f * (u - 0.5f);
		    float g = 128f + 128f * (v - 0.5f);
		    float b = 128f + 128f * (0.5f - u);
		    if(touching) {
		    	canvas.drawARGB(255, (int) r, (int) g, (int) b);
		    }
            
            // Draw the "strings".
            paint.setStyle(Style.STROKE);
            paint.setStrokeWidth(1.5f);
            if (sw > 0 && nx > 0 && colsX != null) {
                final float nw = sw / nx;
                for (int s = 0; s < nx; ++s) {
                    float x = s * nw + nw / 2f;
                    paint.setColor(colsX[s % colsX.length]);
                    canvas.drawLine(x, 0, x, sh, paint);
                }
            }
            if (sh > 0 && ny > 0 && colsY != null) {
                final float nh = sh / ny;
                for (int s = 0; s < ny; ++s) {
                    float y = s * nh + nh / 2f;
                    y = sh - y;
                    paint.setColor(colsY[s % colsY.length]);
                    canvas.drawLine(0, y, sw, y, paint);
                }
            }
            
            if(touching && scale != null) {
            	// 	Draw the user's finger.
            	paint.setStyle(Style.FILL);
            	paint.setColor(0x80ffffff);
            	paint.setAntiAlias(true);
            
            	float cx = (sw*(0.5f+(int)(u*nx)))/nx;
            	float cy = duet?((sh*(0.5f+(int)((1-v)*ny)))/ny):((1-v)*sh);
            	canvas.drawCircle(cx, cy, sw/(2*nx), paint);
            }
		}
		
		private int noteColor(float n) {
            int a = 255;
            hsv[0] = n / 12f * 300f;
            hsv[1] = n == 0 ? 0 : 1;
            hsv[2] = 1;
            return Color.HSVToColor(a, hsv);
		}
		
	}
}