package as.adamsmith.etherealdialpad;

import java.util.List;

import android.app.Activity;
import android.content.ComponentName;
import android.content.Intent;
import android.content.pm.ActivityInfo;
import android.content.pm.ResolveInfo;
import android.content.pm.PackageManager.NameNotFoundException;
import android.content.res.Resources;
import android.net.Uri;
import android.os.Bundle;
import android.util.Log;
import android.view.LayoutInflater;
import android.view.Menu;
import android.view.MenuItem;
import android.view.View;
import android.view.ViewGroup;
import android.view.Window;
import android.widget.AdapterView;
import android.widget.ArrayAdapter;
import android.widget.ImageView;
import android.widget.ListView;
import android.widget.TextView;

public class Main extends Activity {
	
	@Override
	public void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
		
		if(BuildChecker.isHoneycombOrBetter()) {
			try {
				java.lang.reflect.Field featureActionBarOverlayField;
				featureActionBarOverlayField = Window.class.getField("FEATURE_ACTION_BAR_OVERLAY");
				int featureActionBarOverlay = featureActionBarOverlayField.getInt(null);
				getWindow().requestFeature(featureActionBarOverlay);
			} catch (Throwable e) {
				e.printStackTrace();
			}
		} else {
			getWindow().requestFeature(Window.FEATURE_NO_TITLE);
		}
		
		setContentView(R.layout.main);
	}
	
	@Override
	public void onResume() {
		super.onResume();
		
		String performAction = getResources().getString(R.string.perform_action);
		
		final List<ResolveInfo> resolveInfos =
			getPackageManager()
				.queryIntentActivities(new Intent(performAction), 0);
		
		ListView padList = (ListView)findViewById(R.id.pad_list);
		padList.setOnItemClickListener(new AdapterView.OnItemClickListener() {
			@Override
			public void onItemClick(AdapterView<?> parent, View view, int pos,
					long id) {
				launchWithPad(resolveInfos.get(pos).activityInfo);
			}
		});
		
		padList.setAdapter(makeListAdapter(resolveInfos));
	}
	
	private ArrayAdapter<ResolveInfo> makeListAdapter(List<ResolveInfo> infos) {
		return new ArrayAdapter<ResolveInfo>(
				this,
				R.layout.pad_element, infos) {
			LayoutInflater inflater = LayoutInflater.from(Main.this);
			@Override
			public View getView(int position, View convertView, ViewGroup parent) {
				View padElement = inflater.inflate(R.layout.pad_element, null);
				ImageView padIcon = (ImageView)padElement.findViewById(R.id.pad_icon);
				TextView padName = (TextView)padElement.findViewById(R.id.pad_name);
				ResolveInfo it = getItem(position);
				
				ComponentName cname = new ComponentName(it.activityInfo.packageName, it.activityInfo.name);
				Resources resources = getContext().getResources();
				try {
					resources = getPackageManager().getResourcesForActivity(cname);
				} catch (NameNotFoundException e) {
					e.printStackTrace();
				}
				if(it.activityInfo.icon != 0) {
					padIcon.setImageDrawable(resources.getDrawable(it.activityInfo.icon));
				} else {
					padIcon.setImageResource(R.drawable.defaultpad_icon);
				}
				
				if(it.activityInfo.labelRes != 0) {
					padName.setText(resources.getString(it.activityInfo.labelRes));
				} else {
					padName.setText(R.string.no_label);
				}
				
				return padElement;
			}
		};
	}
	
	private void launchWithPad(ActivityInfo activityInfo) {
		Log.d(getClass().getSimpleName(), "I want to launch " + activityInfo.name + " from " + activityInfo.packageName);
		Intent launchPad = new Intent()
			.setComponent(new ComponentName(activityInfo.packageName, activityInfo.name));
		startActivity(launchPad);
	}

	
	@Override
    public boolean onCreateOptionsMenu(Menu menu) {
    	getMenuInflater().inflate(R.menu.main_menu, menu);
    	
    	if(BuildChecker.isHoneycombOrBetter()) {
    		try {
				java.lang.reflect.Method getActionBar = getClass().getMethod("getActionBar");
				Object actionBar = getActionBar.invoke(this);
				java.lang.reflect.Method setDisplayShowHomeEnabled = actionBar.getClass().getMethod("setDisplayShowHomeEnabled", new Class[] {boolean.class});
				setDisplayShowHomeEnabled.invoke(actionBar, false);
				java.lang.reflect.Method setDisplayShowTitleEnabled = actionBar.getClass().getMethod("setDisplayShowTitleEnabled", new Class[] {boolean.class});
				setDisplayShowTitleEnabled.invoke(actionBar, false);
    		} catch (Throwable e) {
				e.printStackTrace();
			}
    	}
    	
    	return true;
    }
    
    @Override
    public boolean onOptionsItemSelected(MenuItem item) {
    	switch(item.getItemId()) {
    	case R.id.synth_preferences_menu:
        	startActivity(new Intent(this, SynthPreferences.class));
        	return true;
    	case R.id.more_dialpads_menu:
    		startActivity(new Intent(Intent.ACTION_VIEW).setData(Uri.parse("market://search?q=\"ethereal dialpad\" plug-in")));
        	return true;
    	}
    	
    	
    	return true;
    }
}