package as.adamsmith.etherealdialpad;

import android.content.Context;
import android.graphics.Canvas;
import android.graphics.Paint;
import android.os.Bundle;
import android.os.RemoteException;
import android.view.MotionEvent;
import android.view.View;

public class PointPad extends Pad {

	@Override
	public void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
		setContentView(new PointView(this));
	}
	
	class PointView extends View {
		final int N = 100;
		
		final Paint spotPaint = new Paint();
		
		public int manualOffset;
		
		long lastTick;
		float x, y;
		float targetX, targetY;
		boolean isTouching;
		
		float lines[];
		
		public PointView(Context context) {
			super(context);

			lines = new float[4*N];
			
			setBackgroundColor(0xFF101020);
			
			spotPaint.setARGB(255, 255,200,64);
			spotPaint.setStyle(Paint.Style.STROKE);
			spotPaint.setStrokeWidth(2);
			spotPaint.setAntiAlias(true);
			
			setKeepScreenOn(true);
			setFocusable(true);
		}
		
		private void updateLines() {
			final float k = 0.975f;
			final float tx = (1.0f - k)*targetX;
			final float ty = (1.0f - k)*targetY;
			final float f = 0.875f;
			final float w = getWidth();
			final float h = getHeight();
			final float d = 0.025f;
			for(int i = 0; i < N; i++) {
				float px = lines[4*i + 0];
				float py = lines[4*i + 1];
				if(isTouching) {
					lines[4*i + 0] = lines[4*i + 2];
					lines[4*i + 1] = lines[4*i + 3];
					lines[4*i + 2] = k*lines[4*i + 2] + tx + (lines[4*i + 2]-px)*f;
					lines[4*i + 3] = k*lines[4*i + 3] + ty + (lines[4*i + 3]-py)*f;
				} else {
					lines[4*i + 0] = lines[4*i + 2];
					lines[4*i + 1] = lines[4*i + 3];
					lines[4*i + 2] += (lines[4*i + 0]-px)*f;
					lines[4*i + 3] += (lines[4*i + 1]-py)*f;
				}
				
				if(Math.random() < 0.1) {
					lines[4*i + 0] = w*(float)Math.random();
					lines[4*i + 1] = h*(float)Math.random();
					lines[4*i + 2] = lines[4*i + 0] + d*w*(float)(Math.random()-0.5);
					lines[4*i + 3] = lines[4*i + 1] + d*h*(float)(Math.random()-0.5);
				}
			}
		}
		
		@Override
	    public boolean onTouchEvent(MotionEvent event) {
			float ex = event.getX();
			float ey = event.getY();
			
			targetX = ex;
			targetY = ey;
			
			x = ex / getWidth();
			y = 1 - ey / getHeight();
			    
	    	if(event.getAction() == MotionEvent.ACTION_MOVE) {
	    		try {
					synthService.primaryXY(x, y);
				} catch (RemoteException e) {
					e.printStackTrace();
				}
	    		
	    	}
	    	
	    	if(event.getAction() == MotionEvent.ACTION_DOWN) {
	    		isTouching = true;
	    		try {
					synthService.primaryOn();
					synthService.primaryXY(x, y);
				} catch (RemoteException e) {
					e.printStackTrace();
				}
	    	}
	    	
	    	if(event.getAction() == MotionEvent.ACTION_UP) {
	    		isTouching = false;
	    		try {
					synthService.primaryOff();
					synthService.primaryXY(x, y);
				} catch (RemoteException e) {
					e.printStackTrace();
				}
	    	}

	    	invalidate();
	    	
	    	return true;
	    }
		
		@Override
		public void onDraw(Canvas canvas) {
			updateLines();
			
			canvas.drawLines(lines, spotPaint);
			
			invalidate();
		}
	}

}